#include "animcontrol.h"
#include "util.h"

IMPLEMENT_CLASS(AnimControl, wxPanel)

BEGIN_EVENT_TABLE(AnimControl, wxPanel)
	EVT_COMBOBOX(ID_ANIM, AnimControl::OnAnim)
	EVT_COMBOBOX(ID_LOOPS, AnimControl::OnLoop)
	EVT_COMBOBOX(ID_SKIN, AnimControl::OnSkin)
	EVT_COMBOBOX(ID_ITEMSET, AnimControl::OnItemSet)

	EVT_CHECKBOX(ID_OLDSTYLE, AnimControl::OnCheck)

	EVT_BUTTON(ID_PLAY, AnimControl::OnButton)
	EVT_BUTTON(ID_PAUSE, AnimControl::OnButton)
	EVT_BUTTON(ID_STOP, AnimControl::OnButton)
	EVT_BUTTON(ID_ADDANIM, AnimControl::OnButton)
	EVT_BUTTON(ID_CLEARANIM, AnimControl::OnButton)
	EVT_BUTTON(ID_PREVANIM, AnimControl::OnButton)
	EVT_BUTTON(ID_NEXTANIM, AnimControl::OnButton)
	EVT_SLIDER(ID_SPEED, AnimControl::OnSliderUpdate)
	EVT_SLIDER(ID_FRAME, AnimControl::OnSliderUpdate)
END_EVENT_TABLE()

AnimControl::AnimControl(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
{
	if(Create(parent, id, pos, size, 0, _T("AnimControlFrame")) == false) {
		wxMessageBox("Failed to create a window for our AnimControl!", "Error");
		return;
	}

	const wxString strLoops[10] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};

	//wxPanel *p = new wxPanel(this, -1, pos, size);
	//wxButton *but = new wxButton(p, ID_PAUSE, _("Pause"), wxPoint(10,10), wxSize(48,24));
	
	animList = new wxComboBox(this, ID_ANIM, _("Animation"), wxPoint(10,10), wxSize(144,16), 0, NULL, wxCB_READONLY, wxDefaultValidator, _("Animation")); //|wxCB_SORT); //wxPoint(66,10)
	loopList = new wxComboBox(this, ID_LOOPS, wxString("0"), wxPoint(154,10), wxSize(40,16), 10, strLoops, wxCB_READONLY, wxDefaultValidator, wxString("Loops")); //|wxCB_SORT); //wxPoint(66,10)
	btnAdd = new wxButton(this, ID_ADDANIM, _("Add"), wxPoint(194,10), wxSize(36,20));

	skinList = new wxComboBox(this, ID_SKIN, _("Skin"), wxPoint(250,10), wxSize(140,16), 0, NULL, wxCB_READONLY);
	skinList->Show(FALSE);
	randomSkins = true;
	defaultDoodads = true;

	wmoList = new wxComboBox(this, ID_ITEMSET, _("Item set"), wxPoint(220,10), wxSize(128,16), 0, NULL, wxCB_READONLY);
	wmoList->Show(FALSE);
	wmoLabel = new wxStaticText(this, -1, _T(""), wxPoint(10,15), wxSize(192,16));
	wmoLabel->Show(FALSE);

	speedSlider = new wxSlider(this, ID_SPEED, 10, 1, 40, wxPoint(360,56), wxSize(100,38), wxSL_AUTOTICKS);
	speedSlider->SetTickFreq(10, 1);
	speedLabel = new wxStaticText(this, -1, _("Speed: 1.0x"), wxPoint(360,40), wxDefaultSize);

	frameLabel = new wxStaticText(this, -1, _("Frame: 0"), wxPoint(250,40), wxDefaultSize);
	frameSlider = new wxSlider(this, ID_FRAME, 1, 1, 10, wxPoint(250,56), wxSize(100,38), wxSL_AUTOTICKS);
	frameSlider->SetTickFreq(2, 1);

	wxButton *btnPlay = new wxButton(this, ID_PLAY, _("Play"), wxPoint(10,40), wxSize(45,20));
	wxButton *btnPause = new wxButton(this, ID_PAUSE, _("Pause"), wxPoint(53,40), wxSize(45,20));
	wxButton *btnStop = new wxButton(this, ID_STOP, _("Stop"), wxPoint(96,40), wxSize(45,20));
	
	wxButton *btnClear = new wxButton(this, ID_CLEARANIM, _("Clear"), wxPoint(10,64), wxSize(45,20));
	wxButton *btnPrev = new wxButton(this, ID_PREVANIM, _T("<<"), wxPoint(53,64), wxSize(45,20));
	wxButton *btnNext = new wxButton(this, ID_NEXTANIM, _T(">>"), wxPoint(96,64), wxSize(45,20));
	
	wxCheckBox *oldStyle = new wxCheckBox(this, ID_OLDSTYLE, _("Auto Animate"), wxPoint(150,40), wxDefaultSize, 0);
	oldStyle->SetValue(true);

	model = NULL;
	wmo = NULL;

	bOldStyle = true;
}

AnimControl::~AnimControl()
{
	//animList->Clear();
	//skinList->Clear();

	/*
	delete wmoList;
	delete wmoLabel;

	delete speedSlider;
	delete speedLabel;

	delete frameSlider;
	delete frameLabel;
	*/
}

bool AnimControl::Init()
{
	if (!animdb.open())
		return false;

	if (!modeldb.open())
		return false;

	if (!skindb.open())
		return false;

	return true;
}


void AnimControl::UpdateModel(Model *m)
{
	if (m == NULL)
		return;

	model = m;
	wmo = 0;

	animList->Clear();
	skinList->Clear();
	int useanim = -1;
	
	if (!model->animated || model->anims == NULL) {
		this->Show(false);
		return;
	} else {
		this->Show(true);
	}

	wxString strName;
	for (unsigned int i=0; i<model->header.nAnimations; i++) {
		const char *name;
		
		try {
			AnimDB::Record rec = animdb.getByAnimID(model->anims[i].animID);
			name = rec.getString(AnimDB::Name);
		} catch (AnimDB::NotFound) {
			name = "???";
		}
		
		// Give our animation list some numbering
		strName << i << ". " << name;
		animList->Append( strName );
		strName = "";
		
		if ((useanim==-1) && (wxString(name)=="Stand")) 
			useanim = i;
	}

	if (useanim==-1)
		useanim = 0;
		//return;

	model->currentAnim = useanim;
	animList->Select(useanim);
	animList->Show(true);
	loopList->Show(true);
	btnAdd->Show(true);

	frameSlider->SetRange(model->anims[useanim].timeStart, model->anims[useanim].timeEnd);
	frameSlider->SetTickFreq(model->anims[useanim].playSpeed, 1);
	//canvas->animManager.Init(model->anims, useanim, 0);
	canvas->model->animManager->Set(0, useanim, 0);

	bool res = false;
	
	if (m->name.substr(0,9) != "Character") {

		if (m->name.substr(0,8) == "Creature")
			res = UpdateCreatureModel(m);

		if (!res) {
			if (m->name.substr(0,4) == "Item")
				res = UpdateItemModel(m);
		}
	}

	skinList->Show(res);

	wmoList->Show(false);
	wmoLabel->Show(false);
}

void AnimControl::UpdateWMO(WMO *w, int group)
{
	bool newwmo = (oldname != w->name);
	oldname = w->name;

	model = 0;
	wmo = w;

	//Model *m = static_cast<Model*>(canvas->root->children[0]);

	//if (!m || m->anims==NULL)
	//	return;

	//m->animManager->Reset();

	frameSlider->SetRange(0, 10);
	frameSlider->SetTickFreq(2, 1);
	//canvas->animManager.Init(NULL, 0, 0);
	
	animList->Show(false);
	skinList->Show(false);
	loopList->Show(false);
	btnAdd->Show(false);
	
	if (newwmo) {
		// build itemset list
		wmoList->Clear();
		wmoList->Append(_T("(No doodads)"));

		for (size_t i=0; i<wmo->doodadsets.size(); i++) {
			wmoList->Append(wxString(wmo->doodadsets[i].name, *wxConvCurrent));
		}

		int sel = defaultDoodads ? 1 : 0;
		wmo->includeDefaultDoodads = defaultDoodads;
		wmoList->Select(sel);
		wmo->showDoodadSet(sel-1);
	}
	wmoList->Show(TRUE);

	// get wmo name or current wmogroup name/descr
	if (group>=-1 && group<wmo->nGroups) {
		std::string label;
		label = w->name.substr(w->name.find_last_of('\\')+1);
		if (group>=0) {
			label += " - " + wmo->groups[group].name;
			if (wmo->groups[group].desc.length()) {
				label += " - " + wmo->groups[group].desc;
			}
		}
		wmoLabel->SetLabel(wxString(label.c_str(), *wxConvCurrent));
	} else {
		wmoLabel->SetLabel(_("This group has been removed from the WMO"));
	}
	wmoLabel->Show(TRUE);
}


bool AnimControl::UpdateCreatureModel(Model *m)
{
	wxString fn(m->name);

	// replace M2 with MDX
	if (fn.Last() == '2') {
		fn[fn.Length()-1] = 'd';
		fn.Append("x");
	}

	// replace CREATURE with Creature
	if (fn.Mid(0,8) == "CREATURE") {
		fn.replace(0,8,"Creature");
	}

	TextureSet skins;

	// see if this model has skins
	try {
		CreatureModelDB::Record rec = modeldb.getByFilename(fn);
		// for character models, don't use skins
		if (rec.getUInt(CreatureModelDB::Type) != 4) {
			//TextureSet skins;
			unsigned int modelid = rec.getUInt(CreatureModelDB::ModelID);

			for (CreatureSkinDB::Iterator it = skindb.begin();  it!=skindb.end();  ++it) {
				if (it->getUInt(CreatureSkinDB::ModelID) == modelid) {
					TextureGroup grp;
					for (int i=0; i<TextureGroup::num; i++) {
						//const char *skin = it->getString(CreatureSkinDB::Skin + i);
						std::string skin(it->getString(CreatureSkinDB::Skin + i));
						
						grp.tex[i] = skin;
					}
					grp.base = 11;
					grp.count = 3;
					if (grp.tex[0].length() > 0) skins.insert(grp);
				}
			}
			
			if (fn == "Creature\\furbolg\\Furbolg.mdx") {
				TextureGroup grp;
				grp.tex[0] = "FurbolgSkinPanda";
				grp.tex[1] = "FurbolgStuffWhite";
				grp.base = 11;
				grp.count = 2;
				skins.insert(grp);
			} else if (fn == "Creature\\Murloc\\BabyMurloc.mdx") {
				TextureGroup grp;
				grp.tex[0] = "MurlocBabyBlue";
				grp.tex[1] = "TopHat";
				grp.tex[2] = "MurlocBabyReflect";
				grp.base = 11;
				grp.count = 3;
				skins.insert(grp);
				grp.tex[0] = "MurlocBabyGreen";
				skins.insert(grp);
				grp.tex[0] = "MurlocBabyOrange";
				skins.insert(grp);
				grp.tex[0] = "MurlocBabyPink";
				skins.insert(grp);
				grp.tex[0] = "MurlocBabyPurple";
				skins.insert(grp);
				grp.tex[0] = "MurlocBabyWhite";
				skins.insert(grp);
			} else if (fn == "Creature\\FelBeast\\FelBeast.mdx") {
				TextureGroup grp;
				grp.tex[0] = "FelBeastSkinGreenBlack";
				//grp.tex[1] = "TopHat";
				//grp.tex[2] = "MurlocBabyReflect";
				grp.base = 11;
				grp.count = 1;
				skins.insert(grp);
			} else if (fn == "Creature\\Gryphon\\Gryphon.mdx") {
				TextureGroup grp;
				grp.tex[0] = "Gryphon_Skin01Black";
				grp.tex[1] = "Gryphon_Skin02Black";
				//grp.tex[2] = "MurlocBabyReflect";
				grp.base = 11;
				grp.count = 2;
				skins.insert(grp);
				grp.tex[0] = "Gryphon_Skin01White";
				grp.tex[1] = "Gryphon_Skin02White";
				skins.insert(grp);
			}

			// hard-coded fixes.
			if (skins.empty() == true) {
				if (fn == "Creature\\DwarfMaleWarriorLight\\DwarfMaleWarriorLight.mdx") {
					TextureGroup grp;
					grp.tex[0] = "DwarfMaleWarriorLightSkin";
					grp.base = 11;
					grp.count = 1;
					skins.insert(grp);
					grp.tex[0] = "DwarfMaleWarriorLightSkin2";
					grp.base = 11;
					grp.count = 1;
					skins.insert(grp);
				}

				if (fn == "Creature\\FaerieDragon\\FaerieDragon_Ghost.mdx") {
					TextureGroup grp;
					grp.tex[0] = "FaerieDragon_Ghost";
					grp.tex[1] = "faeriewing_Ghost";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
				}

				if (fn == "Creature\\FelBat\\FelBat.mdx") {
					TextureGroup grp;
					grp.tex[0] = "BatSkin01";
					grp.tex[1] = "BatSkin02";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
					grp.tex[0] = "BatSkinBrown01";
					grp.tex[1] = "BatSkinBrown02";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
					grp.tex[0] = "BatSkinViolet01";
					grp.tex[1] = "BatSkinViolet02";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
					grp.tex[0] = "BatSkinWhite01";
					grp.tex[1] = "BatSkinWhite02";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
				}

				if (fn == "Creature\\HighElf\\HighElfMale_Priest.mdx") {
					TextureGroup grp;
					grp.tex[0] = "HighElfMalePriest";
					grp.base = 11;
					grp.count = 1;
					skins.insert(grp);
				}
				
				if (fn == "Creature\\OrcMaleWarriorHeavy\\OrcMaleWarriorHeavy.mdx") {
					TextureGroup grp;
					grp.tex[1] = "OrcMaleSamuHairSkin";
					grp.tex[0] = "OrcNPCSkin";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
				}
				
				if (fn == "Creature\\SeaGiant\\SeaGiant.mdx") {
					TextureGroup grp;
					grp.tex[0] = "SeaGiantSkin01";
					grp.tex[1] = "SeaGiantSkin02";
					grp.base = 11;
					grp.count = 2;
					skins.insert(grp);
				}

				if (fn == "Creature\\SeaLion\\SeaLion.mdx") {
					TextureGroup grp;
					grp.tex[0] = "SeaLionSkin";
					grp.base = 11;
					grp.count = 1;
					skins.insert(grp);
				}

				if (fn == "Creature\\StormCrow\\StormCrow.mdx") {
					TextureGroup grp;
					grp.tex[0] = "StormCrowSkin";
					grp.base = 11;
					grp.count = 1;
					skins.insert(grp);
				}


			}
			
			return FillSkinSelector(skins);

		} else return false;
	} catch (CreatureModelDB::NotFound) {
		// Try hardcoding some fixes for missing model info from the DBC
		if(fn == "Creature\\DwarfMaleWarriorLight\\DwarfMaleWarriorLight_Ghost.mdx") {
			TextureGroup grp;
			grp.tex[0] = "DwarfMaleWarriorLightSkin_Ghost";
			grp.base = 11;
			grp.count = 1;
			skins.insert(grp);
			return FillSkinSelector(skins);
		}

		if(fn == "Creature\\MountedDemonKnight\\MountedDemonKnight.mdx") {
			TextureGroup grp;
			grp.tex[0] = "MountedDemonKnightBlack_01";
			grp.tex[1] = "MountedDemonKnightBlack_02";
			grp.tex[2] = "MountedDemonKnightBlack_03";
			grp.base = 11;
			grp.count = 3;
			skins.insert(grp);
			return FillSkinSelector(skins);
		}

		if(fn == "Creature\\OrcFemaleWarriorLight\\OrcFemale.mdx") {
			TextureGroup grp;
			grp.tex[0] = "OrcFemaleSkin";
			grp.tex[1] = "OrcFemaleSkin";
			grp.base = 11;
			grp.count = 2;
			skins.insert(grp);
			return FillSkinSelector(skins);
		}

		if(fn == "Creature\\ReinDeer\\ReinDeer.mdx") {
			TextureGroup grp;
			grp.tex[0] = "ReinDeer";
			grp.base = 11;
			grp.count = 1;
			skins.insert(grp);
			return FillSkinSelector(skins);
		} 
		
		if(fn == "Creature\\ReinDeer\\ReinDeerMount.mdx") {
			TextureGroup grp;
			grp.tex[0] = "ReinDeerMount";
			grp.base = 11;
			grp.count = 1;
			skins.insert(grp);
			return FillSkinSelector(skins);
		}

		return false;
	}

}

bool AnimControl::UpdateItemModel(Model *m)
{
	std::string fn = m->name;
	if (fn[fn.length()-1]=='2') {
		fn[fn.length()-1] = 'd';
		fn.append("x");
	}
	size_t p = fn.find_last_of('\\');
	fn = fn.substr(p+1);

	TextureSet skins;

	for (ItemDisplayDB::Iterator it = itemdb->begin(); it != itemdb->end(); ++it) {
		if (!strcmp(it->getString(ItemDisplayDB::Model), fn.c_str())) {
            TextureGroup grp;
			grp.base = 2;
			grp.count = 1;
			const char *skin = it->getString(ItemDisplayDB::Skin);
			grp.tex[0] = skin;
			if (grp.tex[0].length() > 0) skins.insert(grp);
		}
		if (!strcmp(it->getString(ItemDisplayDB::Model2), fn.c_str())) {
            TextureGroup grp;
			grp.base = 2;
			grp.count = 1;
			const char *skin = it->getString(ItemDisplayDB::Skin2);
			grp.tex[0] = skin;
			if (grp.tex[0].length() > 0) skins.insert(grp);
		}
	}

	return FillSkinSelector(skins);
}


bool AnimControl::FillSkinSelector(TextureSet &skins)
{
	if (skins.size() > 0) {
		int num = 0;
		// fill our skin selector
		for (TextureSet::iterator it = skins.begin(); it != skins.end(); ++it) {
			skinList->Append(wxString(it->tex[0].c_str(), *wxConvCurrent));
			TextureGroup *grp = new TextureGroup(*it);
			skinList->SetClientData(num++, grp);
		}

		int mySkin = randomSkins ? randint(0,(int)skins.size()-1) : 0;
		SetSkin(mySkin);

		skinList->Select(mySkin);
		return (skins.size() > 1);
	} else return false;
}


void AnimControl::OnButton(wxCommandEvent &event)
{
	if (!model)
		return;

	int id = event.GetId();

	if (id == ID_PLAY) {
		canvas->model->currentAnim = canvas->model->animManager->GetAnim();
		canvas->model->animManager->Play();
	} else if (id == ID_PAUSE) {
		canvas->model->animManager->Pause();
	} else if (id == ID_STOP) {
		canvas->model->animManager->Stop();
	} else if (id == ID_CLEARANIM) {
		canvas->model->animManager->Clear();
	} else if (id == ID_ADDANIM) {
		canvas->model->animManager->AddAnim(selectedAnim, loopList->GetSelection());
	} else if (id == ID_PREVANIM) {
		canvas->model->animManager->PrevFrame();
		SetAnimFrame(canvas->model->animManager->GetFrame());
	} else if (id == ID_NEXTANIM) {
		canvas->model->animManager->NextFrame();
		SetAnimFrame(canvas->model->animManager->GetFrame());
	}
}

void AnimControl::OnCheck(wxCommandEvent &event)
{
	if (event.GetId() == ID_OLDSTYLE)
		bOldStyle = event.IsChecked();
}

void AnimControl::OnAnim(wxCommandEvent &event)
{
	if (model) {
		selectedAnim = event.GetSelection();

		if (bOldStyle == true) {
			model->currentAnim = selectedAnim;
			canvas->model->animManager->Stop();
			canvas->model->animManager->Set(0, selectedAnim, loopList->GetSelection());
			canvas->model->animManager->Play();
			
			frameSlider->SetRange(model->anims[selectedAnim].timeStart, model->anims[selectedAnim].timeEnd);
			frameSlider->SetTickFreq(model->anims[selectedAnim].playSpeed, 1);
		}
	}

	//canvas->resetTime();
}

void AnimControl::OnSkin(wxCommandEvent &event)
{
	if (model) {
		int sel = event.GetSelection();
		SetSkin(sel);
	}
}

void AnimControl::OnItemSet(wxCommandEvent &event)
{
	if (wmo) {
		int sel = event.GetSelection();
		// -1 for no doodads
		wmo->showDoodadSet(sel - 1);
	}
}

void AnimControl::OnSliderUpdate(wxCommandEvent &event)
{
	if (event.GetId() == ID_SPEED)
		SetAnimSpeed(speedSlider->GetValue() / 10.0f);
	else if (event.GetId() == ID_FRAME)
		SetAnimFrame(frameSlider->GetValue());

}

void AnimControl::OnLoop(wxCommandEvent &event)
{
	if (bOldStyle == true) {
		canvas->model->animManager->Stop();
		canvas->model->animManager->Set(0, selectedAnim, loopList->GetSelection());
		canvas->model->animManager->Play();
	} 
}

void AnimControl::SetSkin(int num)
{
	TextureGroup *grp = (TextureGroup*) skinList->GetClientData(num);
	for (int i=0; i<grp->count; i++) {
		if (model->useReplaceTextures[grp->base+i]) {
			texturemanager.del(model->replaceTextures[grp->base+i]);
			TextureID def = texturemanager.add(makeSkinTexture(model->name.c_str(),grp->tex[i].c_str()));
			model->replaceTextures[grp->base+i] = def;
		}
	}
}

std::string AnimControl::makeSkinTexture(const char *texfn, const char *skin)
{
	std::string res = texfn;
	size_t i = res.find_last_of('\\');
	res = res.substr(0,i+1);
	res.append(skin);
	res.append(".blp");
	return res;
}

void AnimControl::SetAnimSpeed(float speed)
{
	animSpeed = speed;
	char buf[64];
	sprintf(buf, _("Speed: %.1fx"), animSpeed);
	speedLabel->SetLabel(wxString(buf,*wxConvCurrent));
}


void AnimControl::SetAnimFrame(int frame)
{
	if (!canvas->model || !canvas->model->animManager)
		return;

	canvas->model->animManager->SetFrame((unsigned int) frame);
	
	int frameNum = (frame - model->anims[model->currentAnim].timeStart);

	char buf[64];
	sprintf(buf, _("Frame: %i"), frameNum);
	frameLabel->SetLabel(wxString(buf,*wxConvCurrent));

	frameSlider->SetValue(frame);
}

